@@ -91,8 +91,8 @@ class Node(_common.ListMixin, resource.Resource):
         is_maintenance='maintenance',
     )
 
-    # The retired and retired_reason fields introduced in 1.61 (Ussuri).
-    _max_microversion = '1.61'
+    # Ability to change boot_mode and secure_boot, introduced in 1.76 (Xena).
+    _max_microversion = '1.76'
 
     # Properties
     #: The UUID of the allocation associated with this node. Added in API
@@ -101,6 +101,8 @@ class Node(_common.ListMixin, resource.Resource):
     #: A string or UUID of the tenant who owns the baremetal node. Added in API
     #: microversion 1.50.
     owner = resource.Body("owner")
+    #: The current boot mode state (uefi/bios). Added in API microversion 1.75.
+    boot_mode = resource.Body("boot_mode")
     #: The UUID of the chassis associated wit this node. Can be empty or None.
     chassis_id = resource.Body("chassis_uuid")
     #: The current clean step.
@@ -148,6 +150,9 @@ class Node(_common.ListMixin, resource.Resource):
     #: Whether the node is marked for retirement. Added in API microversion
     #: 1.61.
     is_retired = resource.Body("retired", type=bool)
+    #: Whether the node is currently booted with secure boot turned on.
+    #: Added in API microversion 1.75.
+    is_secure_boot = resource.Body("secure_boot", type=bool)
     #: Any error from the most recent transaction that started but failed to
     #: finish.
     last_error = resource.Body("last_error")
@@ -346,7 +351,7 @@ class Node(_common.ListMixin, resource.Resource):
 
     def set_provision_state(self, session, target, config_drive=None,
                             clean_steps=None, rescue_password=None,
-                            wait=False, timeout=None):
+                            wait=False, timeout=None, deploy_steps=None):
         """Run an action modifying this node's provision state.
 
         This call is asynchronous, it will return success as soon as the Bare
@@ -366,10 +371,13 @@ class Node(_common.ListMixin, resource.Resource):
         :param wait: Whether to wait for the target state to be reached.
         :param timeout: Timeout (in seconds) to wait for the target state to be
             reached. If ``None``, wait without timeout.
+        :param deploy_steps: Deploy steps to execute, only valid for ``active``
+            and ``rebuild`` target.
 
         :return: This :class:`Node` instance.
-        :raises: ValueError if ``config_drive``, ``clean_steps`` or
-            ``rescue_password`` are provided with an invalid ``target``.
+        :raises: ValueError if ``config_drive``, ``clean_steps``,
+            ``deploy_steps`` or ``rescue_password`` are provided with an
+            invalid ``target``.
         :raises: :class:`~openstack.exceptions.ResourceFailure` if the node
             reaches an error state while waiting for the state.
         :raises: :class:`~openstack.exceptions.ResourceTimeout` if timeout
@@ -388,6 +396,9 @@ class Node(_common.ListMixin, resource.Resource):
             elif target == 'rebuild':
                 version = _common.CONFIG_DRIVE_REBUILD_VERSION
 
+        if deploy_steps:
+            version = _common.DEPLOY_STEPS_VERSION
+
         version = self._assert_microversion_for(session, 'commit', version)
 
         body = {'target': target}
@@ -404,6 +415,12 @@ class Node(_common.ListMixin, resource.Resource):
                                  '"clean" target')
             body['clean_steps'] = clean_steps
 
+        if deploy_steps is not None:
+            if target not in ('active', 'rebuild'):
+                raise ValueError('Deploy steps can only be provided with '
+                                 '"deploy" and "rebuild" target')
+            body['deploy_steps'] = deploy_steps
+
         if rescue_password is not None:
             if target != 'rescue':
                 raise ValueError('Rescue password can only be provided with '
@@ -818,7 +835,6 @@ class Node(_common.ListMixin, resource.Resource):
         :param boot_device: Boot device to assign to the node.
         :param persistent: If the boot device change is maintained after node
             reboot
-        :return: The updated :class:`~openstack.baremetal.v1.node.Node`
         """
         session = self._get_session(session)
         version = self._get_microversion_for(session, 'commit')
@@ -836,12 +852,72 @@ class Node(_common.ListMixin, resource.Resource):
                .format(node=self.id))
         exceptions.raise_from_response(response, error_message=msg)
 
+    def set_boot_mode(self, session, target):
+        """Make a request to change node's boot mode
+
+        This call is asynchronous, it will return success as soon as the Bare
+        Metal service acknowledges the request.
+
+        :param session: The session to use for making this request.
+        :param target: Boot mode to set for node, one of either 'uefi'/'bios'.
+        :raises: ValueError if ``target`` is not one of 'uefi or 'bios'.
+        """
+        session = self._get_session(session)
+        version = utils.pick_microversion(session,
+                                          _common.CHANGE_BOOT_MODE_VERSION)
+        request = self._prepare_request(requires_id=True)
+        request.url = utils.urljoin(request.url, 'states', 'boot_mode')
+        if target not in ('uefi', 'bios'):
+            raise ValueError("Unrecognized boot mode %s."
+                             "Boot mode should be one of 'uefi' or 'bios'."
+                             % target)
+        body = {'target': target}
+
+        response = session.put(
+            request.url, json=body,
+            headers=request.headers, microversion=version,
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
+
+        msg = ("Failed to change boot mode for node {node}"
+               .format(node=self.id))
+        exceptions.raise_from_response(response, error_message=msg)
+
+    def set_secure_boot(self, session, target):
+        """Make a request to change node's secure boot state
+
+        This call is asynchronous, it will return success as soon as the Bare
+        Metal service acknowledges the request.
+
+        :param session: The session to use for making this request.
+        :param bool target: Boolean indicating secure boot state to set.
+            True/False corresponding to 'on'/'off' respectively.
+        :raises: ValueError if ``target`` is not boolean.
+        """
+        session = self._get_session(session)
+        version = utils.pick_microversion(session,
+                                          _common.CHANGE_BOOT_MODE_VERSION)
+        request = self._prepare_request(requires_id=True)
+        request.url = utils.urljoin(request.url, 'states', 'secure_boot')
+        if not isinstance(target, bool):
+            raise ValueError("Invalid target %s. It should be True or False "
+                             "corresponding to secure boot state 'on' or 'off'"
+                             % target)
+        body = {'target': target}
+
+        response = session.put(
+            request.url, json=body,
+            headers=request.headers, microversion=version,
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
+
+        msg = ("Failed to change secure boot state for {node}"
+               .format(node=self.id))
+        exceptions.raise_from_response(response, error_message=msg)
+
     def add_trait(self, session, trait):
         """Add a trait to a node.
 
         :param session: The session to use for making this request.
         :param trait: The trait to add to the node.
-        :returns: The updated :class:`~openstack.baremetal.v1.node.Node`
         """
         session = self._get_session(session)
         version = utils.pick_microversion(session, '1.37')
@@ -867,7 +943,8 @@ class Node(_common.ListMixin, resource.Resource):
             :class:`~openstack.exceptions.ResourceNotFound` will be
             raised when the trait does not exist.
             Otherwise, ``False`` is returned.
-        :returns: The updated :class:`~openstack.baremetal.v1.node.Node`
+        :returns bool: True on success removing the trait.
+            False when the trait does not exist already.
         """
         session = self._get_session(session)
         version = utils.pick_microversion(session, '1.37')
@@ -878,7 +955,7 @@ class Node(_common.ListMixin, resource.Resource):
             request.url, headers=request.headers, microversion=version,
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
 
-        if ignore_missing or response.status_code == 400:
+        if ignore_missing and response.status_code == 400:
             session.log.debug(
                 'Trait %(trait)s was already removed from node %(node)s',
                 {'trait': trait, 'node': self.id})
@@ -888,7 +965,8 @@ class Node(_common.ListMixin, resource.Resource):
                .format(node=self.id, trait=trait))
         exceptions.raise_from_response(response, error_message=msg)
 
-        self.traits = list(set(self.traits) - {trait})
+        if self.traits:
+            self.traits = list(set(self.traits) - {trait})
 
         return True
 
@@ -900,7 +978,6 @@ class Node(_common.ListMixin, resource.Resource):
 
         :param session: The session to use for making this request.
         :param traits: list of traits to add to the node.
-        :returns: The updated :class:`~openstack.baremetal.v1.node.Node`
         """
         session = self._get_session(session)
         version = utils.pick_microversion(session, '1.37')
@@ -920,6 +997,55 @@ class Node(_common.ListMixin, resource.Resource):
 
         self.traits = traits
 
+    def call_vendor_passthru(self, session, verb, method, body=None):
+        """Call a vendor passthru method.
+
+        :param session: The session to use for making this request.
+        :param verb: The HTTP verb, one of GET, SET, POST, DELETE.
+        :param method: The method to call using vendor_passthru.
+        :param body: The JSON body in the HTTP call.
+        :returns: The HTTP response.
+        """
+        session = self._get_session(session)
+        version = self._get_microversion_for(session, 'commit')
+        request = self._prepare_request(requires_id=True)
+        request.url = utils.urljoin(request.url, 'vendor_passthru?method={}'
+                                    .format(method))
+
+        call = getattr(session, verb.lower())
+        response = call(
+            request.url, json=body,
+            headers=request.headers, microversion=version,
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
+
+        msg = ("Failed to call vendor_passthru for node {node}, verb {verb}"
+               " and method {method}"
+               .format(node=self.id, verb=verb, method=method))
+        exceptions.raise_from_response(response, error_message=msg)
+
+        return response
+
+    def list_vendor_passthru(self, session):
+        """List vendor passthru methods.
+
+        :param session: The session to use for making this request.
+        :returns: The HTTP response.
+        """
+        session = self._get_session(session)
+        version = self._get_microversion_for(session, 'fetch')
+        request = self._prepare_request(requires_id=True)
+        request.url = utils.urljoin(request.url, 'vendor_passthru/methods')
+
+        response = session.get(
+            request.url, headers=request.headers, microversion=version,
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
+
+        msg = ("Failed to list vendor_passthru methods for node {node}"
+               .format(node=self.id))
+        exceptions.raise_from_response(response, error_message=msg)
+
+        return response.json()
+
     def patch(self, session, patch=None, prepend_key=True, has_body=True,
               retry_on_conflict=None, base_path=None, reset_interfaces=None):
 
