@@ -16,31 +16,24 @@ import contextlib
 import fnmatch
 import functools
 import inspect
-import jmespath
-import munch
-import netifaces
 import re
-import six
-import sre_constants
-import sys
 import time
 import uuid
 
 from decorator import decorator
+import jmespath
+import munch
+import netifaces
+import sre_constants
 
 from openstack import _log
 from openstack.cloud import exc
 from openstack.cloud import meta
 
+
 _decorated_methods = []
 
 
-def _exc_clear():
-    """Because sys.exc_clear is gone in py3 and is not in six."""
-    if sys.version_info[0] == 2:
-        sys.exc_clear()
-
-
 def _make_unicode(input):
     """Turn an input into unicode unconditionally
 
@@ -134,7 +127,7 @@ def _filter_list(data, name_or_id, filters):
     if not filters:
         return data
 
-    if isinstance(filters, six.string_types):
+    if isinstance(filters, str):
         return jmespath.search(filters, data)
 
     def _dict_filter(f, d):
@@ -543,8 +536,8 @@ def _call_client_and_retry(client, url, retry_on=None,
         except exc.OpenStackCloudHTTPError as e:
             if (retry_on is not None
                     and e.response.status_code in retry_on):
-                log.debug('Received retryable error {err}, waiting '
-                          '{wait} seconds to retry', {
+                log.debug('Received retryable error %(err)s, waiting '
+                          '%(wait)s seconds to retry', {
                               'err': e.response.status_code,
                               'wait': retry_wait
                           })
@@ -675,7 +668,7 @@ def generate_patches_from_kwargs(operation, **kwargs):
     return sorted(patches)
 
 
-class FileSegment(object):
+class FileSegment:
     """File-like object to pass to requests."""
 
     def __init__(self, filename, offset, length):
