@@ -27,13 +27,26 @@ class VolumeAttachment(resource.Resource):
 
     _query_mapping = resource.QueryParameters("limit", "offset")
 
+    #: The ID for the server.
+    server_id = resource.URI('server_id')
     #: Name of the device such as, /dev/vdb.
     device = resource.Body('device')
     #: The ID of the attachment.
     id = resource.Body('id')
-    #: The ID for the server.
-    server_id = resource.URI('server_id')
+    # FIXME(stephenfin): This conflicts since there is a server ID in the URI
+    # *and* in the body. We need a field that handles both or we need to use
+    # different names.
+    # #: The UUID of the server
+    # server_id = resource.Body('server_uuid')
     #: The ID of the attached volume.
     volume_id = resource.Body('volumeId')
-    #: The ID of the attachment you want to delete or update.
+    #: The UUID of the associated volume attachment in Cinder.
     attachment_id = resource.Body('attachment_id', alternate_id=True)
+    #: The ID of the block device mapping record for the attachment
+    bdm_id = resource.Body('bdm_uuid')
+    #: Virtual device tags for the attachment.
+    tag = resource.Body('tag')
+    #: Indicates whether to delete the volume when server is destroyed
+    delete_on_termination = resource.Body('delete_on_termination')
+    # attachment_id (in responses) and bdm_id introduced in 2.89
+    _max_microversion = '2.89'
