@@ -10,13 +10,15 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import urllib
+from urllib.parse import urlparse
+
 try:
     import simplejson
     JSONDecodeError = simplejson.scanner.JSONDecodeError
 except ImportError:
     JSONDecodeError = ValueError
-from six.moves import urllib
-
+import iso8601
 from keystoneauth1 import adapter
 
 from openstack import _log
@@ -91,15 +93,23 @@ class Proxy(adapter.Adapter):
             if conn:
                 # Per-request setting should take precedence
                 global_request_id = conn._global_request_id
-        response = super(Proxy, self).request(
-            url, method,
-            connect_retries=connect_retries, raise_exc=raise_exc,
-            global_request_id=global_request_id,
-            **kwargs)
-        for h in response.history:
-            self._report_stats(h)
-        self._report_stats(response)
-        return response
+        try:
+            response = super(Proxy, self).request(
+                url, method,
+                connect_retries=connect_retries, raise_exc=raise_exc,
+                global_request_id=global_request_id,
+                **kwargs)
+            for h in response.history:
+                self._report_stats(h)
+            self._report_stats(response)
+            return response
+        except Exception as e:
+            # If we want metrics to be generated we also need to generate some
+            # in case of exceptions as well, so that timeouts and connection
+            # problems (especially when called from ansible) are being
+            # generated as well.
+            self._report_stats(None, url, method, e)
+            raise
 
     def _extract_name(self, url, service_type=None, project_id=None):
         '''Produce a key name to use in logging/metrics from the URL path.
@@ -185,58 +195,94 @@ class Proxy(adapter.Adapter):
 
         return name_parts
 
-    def _report_stats(self, response):
+    def _report_stats(self, response, url=None, method=None, exc=None):
         if self._statsd_client:
-            self._report_stats_statsd(response)
+            self._report_stats_statsd(response, url, method, exc)
         if self._prometheus_counter and self._prometheus_histogram:
-            self._report_stats_prometheus(response)
+            self._report_stats_prometheus(response, url, method, exc)
         if self._influxdb_client:
-            self._report_stats_influxdb(response)
+            self._report_stats_influxdb(response, url, method, exc)
 
-    def _report_stats_statsd(self, response):
-        name_parts = self._extract_name(response.request.url,
+    def _report_stats_statsd(self, response, url=None, method=None, exc=None):
+        if response is not None and not url:
+            url = response.request.url
+        if response is not None and not method:
+            method = response.request.method
+        name_parts = self._extract_name(url,
                                         self.service_type,
                                         self.session.get_project_id())
         key = '.'.join(
-            [self._statsd_prefix, self.service_type, response.request.method]
+            [self._statsd_prefix, self.service_type, method]
             + name_parts)
-        self._statsd_client.timing(key, int(
-            response.elapsed.microseconds / 1000))
-        self._statsd_client.incr(key)
+        with self._statsd_client.pipeline() as pipe:
+            if response is not None:
+                pipe.timing(key, response.elapsed)
+                pipe.incr(key)
+            elif exc is not None:
+                pipe.incr('%s.failed' % key)
 
-    def _report_stats_prometheus(self, response):
-        labels = dict(
-            method=response.request.method,
-            endpoint=response.request.url,
-            service_type=self.service_type,
-            status_code=response.status_code,
-        )
-        self._prometheus_counter.labels(**labels).inc()
-        self._prometheus_histogram.labels(**labels).observe(
-            response.elapsed.microseconds / 1000)
+    def _report_stats_prometheus(self, response, url=None, method=None,
+                                 exc=None):
+        if response is not None and not url:
+            url = response.request.url
+        if response is not None and not method:
+            method = response.request.method
+        parsed_url = urlparse(url)
+        endpoint = "{}://{}{}".format(
+            parsed_url.scheme, parsed_url.netloc, parsed_url.path)
+        if response is not None:
+            labels = dict(
+                method=method,
+                endpoint=endpoint,
+                service_type=self.service_type,
+                status_code=response.status_code,
+            )
+            self._prometheus_counter.labels(**labels).inc()
+            self._prometheus_histogram.labels(**labels).observe(
+                response.elapsed.total_seconds() * 1000)
 
-    def _report_stats_influxdb(self, response):
+    def _report_stats_influxdb(self, response, url=None, method=None,
+                               exc=None):
         # NOTE(gtema): status_code is saved both as tag and field to give
         # ability showing it as a value and not only as a legend.
         # However Influx is not ok with having same name in tags and fields,
         # therefore use different names.
+        if response is not None and not url:
+            url = response.request.url
+        if response is not None and not method:
+            method = response.request.method
+        tags = dict(
+            method=method,
+            name='_'.join(self._extract_name(
+                url, self.service_type,
+                self.session.get_project_id()))
+        )
+        fields = dict(
+            attempted=1
+        )
+        if response is not None:
+            fields['duration'] = int(response.elapsed.total_seconds() * 1000)
+            tags['status_code'] = str(response.status_code)
+            # Note(gtema): emit also status_code as a value (counter)
+            fields[str(response.status_code)] = 1
+            fields['%s.%s' % (method, response.status_code)] = 1
+            # Note(gtema): status_code field itself is also very helpful on the
+            # graphs to show what was the code, instead of counting its
+            # occurences
+            fields['status_code_val'] = response.status_code
+        elif exc:
+            fields['failed'] = 1
+        if 'additional_metric_tags' in self._influxdb_config:
+            tags.update(self._influxdb_config['additional_metric_tags'])
+        measurement = self._influxdb_config.get(
+            'measurement', 'openstack_api') \
+            if self._influxdb_config else 'openstack_api'
+        # Note(gtema) append service name into the measurement name
+        measurement = '%s.%s' % (measurement, self.service_type)
         data = [dict(
-            measurement=(self._influxdb_config.get('measurement',
-                                                   'openstack_api')
-                         if self._influxdb_config else 'openstack_api'),
-            tags=dict(
-                method=response.request.method,
-                service_type=self.service_type,
-                status_code=response.status_code,
-                name='_'.join(self._extract_name(
-                    response.request.url, self.service_type,
-                    self.session.get_project_id())
-                )
-            ),
-            fields=dict(
-                duration=int(response.elapsed.microseconds / 1000),
-                status_code_val=int(response.status_code)
-            )
+            measurement=measurement,
+            tags=tags,
+            fields=fields
         )]
         try:
             self._influxdb_client.write_points(data)
@@ -264,14 +310,13 @@ class Proxy(adapter.Adapter):
         """Get a resource object to work on
 
         :param resource_type: The type of resource to operate on. This should
-                              be a subclass of
-                              :class:`~openstack.resource.Resource` with a
-                              ``from_id`` method.
+            be a subclass of :class:`~openstack.resource.Resource` with a
+            ``from_id`` method.
         :param value: The ID of a resource or an object of ``resource_type``
-                      class if using an existing instance, or ``munch.Munch``,
-                      or None to create a new instance.
-        :param path_args: A dict containing arguments for forming the request
-                          URL, if needed.
+            class if using an existing instance, or ``munch.Munch``,
+            or None to create a new instance.
+        :param attrs: A dict containing arguments for forming the request
+            URL, if needed.
         """
         conn = self._get_connection()
         if value is None:
@@ -328,6 +373,7 @@ class Proxy(adapter.Adapter):
                                   ignore_missing=ignore_missing,
                                   **attrs)
 
+    # TODO(stephenfin): Update docstring for attrs since it's a lie
     @_check_resource(strict=False)
     def _delete(self, resource_type, value, ignore_missing=True, **attrs):
         """Delete a resource
@@ -466,16 +512,13 @@ class Proxy(adapter.Adapter):
             error_message="No {resource_type} found for {value}".format(
                 resource_type=resource_type.__name__, value=value))
 
-    def _list(self, resource_type, value=None,
+    def _list(self, resource_type,
               paginated=True, base_path=None, **attrs):
         """List a resource
 
-        :param resource_type: The type of resource to delete. This should
+        :param resource_type: The type of resource to list. This should
                               be a :class:`~openstack.resource.Resource`
                               subclass with a ``from_id`` method.
-        :param value: The resource to list. It can be the ID of a resource, or
-                      a :class:`~openstack.resource.Resource` object. When set
-                      to None, a new bare resource is created.
         :param bool paginated: When set to ``False``, expect all of the data
                                to be returned in one response. When set to
                                ``True``, the resource supports data being
@@ -524,18 +567,79 @@ class Proxy(adapter.Adapter):
     def _get_cleanup_dependencies(self):
         return None
 
-    def _service_cleanup(self, dry_run=True, status_queue=None):
+    def _service_cleanup(self, dry_run=True, client_status_queue=None,
+                         identified_resources=None, filters=None,
+                         resource_evaluation_fn=None):
         return None
 
     def _service_cleanup_del_res(self, del_fn, obj, dry_run=True,
-                                 status_queue=None):
-        if status_queue:
-            status_queue.put(obj)
-        if not dry_run:
-            try:
-                del_fn(obj)
-            except exceptions.SDKException as e:
-                self.log.error('Cannot delete resource %s: %s', obj, str(e))
+                                 client_status_queue=None,
+                                 identified_resources=None,
+                                 filters=None,
+                                 resource_evaluation_fn=None):
+        need_delete = False
+        try:
+            if (
+                resource_evaluation_fn
+                and callable(resource_evaluation_fn)
+            ):
+                # Ask a user-provided evaluation function if we need to delete
+                # the resource
+                need_del = resource_evaluation_fn(obj, filters,
+                                                  identified_resources)
+                if isinstance(need_del, bool):
+                    # Just double check function returned bool
+                    need_delete = need_del
+            else:
+                need_delete = \
+                    self._service_cleanup_resource_filters_evaluation(
+                        obj,
+                        filters=filters)
+
+            if need_delete:
+                if client_status_queue:
+                    # Put into queue for client status info
+                    client_status_queue.put(obj)
+                if identified_resources is not None:
+                    # Put into internal dict shared between threads so that
+                    # other services might know which other resources were
+                    # identified
+                    identified_resources[obj.id] = obj
+                if not dry_run:
+                    del_fn(obj)
+        except Exception as e:
+            self.log.exception('Cannot delete resource %s: %s', obj, str(e))
+        return need_delete
+
+    def _service_cleanup_resource_filters_evaluation(self, obj, filters=None):
+        part_cond = []
+        if filters is not None and isinstance(filters, dict):
+            for k, v in filters.items():
+                try:
+                    res_val = None
+                    if k == 'created_at' and hasattr(obj, 'created_at'):
+                        res_val = getattr(obj, 'created_at')
+                    if k == 'updated_at' and hasattr(obj, 'updated_at'):
+                        res_val = getattr(obj, 'updated_at')
+                    if res_val:
+                        res_date = iso8601.parse_date(res_val)
+                        cmp_date = iso8601.parse_date(v)
+                        if res_date and cmp_date and res_date <= cmp_date:
+                            part_cond.append(True)
+                        else:
+                            part_cond.append(False)
+                    else:
+                        # There are filters set, but we can't get required
+                        # attribute, so skip the resource
+                        self.log.debug('Requested cleanup attribute %s is not '
+                                       'available on the resource' % k)
+                        part_cond.append(False)
+                except Exception:
+                    self.log.exception('Error during condition evaluation')
+        if all(part_cond):
+            return True
+        else:
+            return False
 
 
 def _json_response(response, result_key=None, error_message=None):
