@@ -10,12 +10,11 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import mock
-import six
-from openstack.tests.unit import base
+from unittest import mock
 
-from openstack.image.v2 import image
 from openstack.compute.v2 import server
+from openstack.image.v2 import image
+from openstack.tests.unit import base
 
 IDENTIFIER = 'IDENTIFIER'
 EXAMPLE = {
@@ -98,13 +97,13 @@ EXAMPLE = {
     'name': 'new-server-test',
     'networks': 'auto',
     'os-extended-volumes:volumes_attached': [],
-    'personality': '28',
     'progress': 0,
     'security_groups': [
         {
             'name': 'default'
         }
     ],
+    'server_groups': ['3caf4187-8010-491f-b6f5-a4a68a40371e'],
     'status': 'ACTIVE',
     'tags': [],
     'tenant_id': '6f70656e737461636b20342065766572',
@@ -207,6 +206,7 @@ class TestServer(base.TestCase):
         self.assertEqual(EXAMPLE['name'], sot.name)
         self.assertEqual(EXAMPLE['progress'], sot.progress)
         self.assertEqual(EXAMPLE['tenant_id'], sot.project_id)
+        self.assertEqual(EXAMPLE['server_groups'], sot.server_groups)
         self.assertEqual(EXAMPLE['status'], sot.status)
         self.assertEqual(EXAMPLE['updated'], sot.updated_at)
         self.assertEqual(EXAMPLE['user_id'], sot.user_id)
@@ -224,7 +224,6 @@ class TestServer(base.TestCase):
                          sot.terminated_at)
         self.assertEqual(EXAMPLE['security_groups'], sot.security_groups)
         self.assertEqual(EXAMPLE['adminPass'], sot.admin_password)
-        self.assertEqual(EXAMPLE['personality'], sot.personality)
         self.assertEqual(EXAMPLE['block_device_mapping_v2'],
                          sot.block_device_mapping)
         self.assertEqual(EXAMPLE['OS-EXT-SRV-ATTR:host'],
@@ -258,7 +257,7 @@ class TestServer(base.TestCase):
         hints = {"hint": 3}
 
         sot = server.Server(id=1, availability_zone=zone, user_data=data,
-                            scheduler_hints=hints)
+                            scheduler_hints=hints, min_count=2, max_count=3)
         request = sot._prepare_request()
 
         self.assertNotIn("OS-EXT-AZ:availability_zone",
@@ -275,6 +274,9 @@ class TestServer(base.TestCase):
                          request.body[sot.resource_key])
         self.assertEqual(request.body["OS-SCH-HNT:scheduler_hints"], hints)
 
+        self.assertEqual(2, request.body[sot.resource_key]['min_count'])
+        self.assertEqual(3, request.body[sot.resource_key]['max_count'])
+
     def test_change_password(self):
         sot = server.Server(**EXAMPLE)
 
@@ -317,8 +319,7 @@ class TestServer(base.TestCase):
                              image='http://image/1', access_ipv4="12.34.56.78",
                              access_ipv6="fe80::100",
                              metadata={"meta var": "meta val"},
-                             personality=[{"path": "/etc/motd",
-                                           "contents": "foo"}])
+                             user_data="ZWNobyAiaGVsbG8gd29ybGQi")
 
         self.assertIsInstance(result, server.Server)
 
@@ -331,7 +332,7 @@ class TestServer(base.TestCase):
                 "accessIPv4": "12.34.56.78",
                 "accessIPv6": "fe80::100",
                 "metadata": {"meta var": "meta val"},
-                "personality": [{"path": "/etc/motd", "contents": "foo"}],
+                "user_data": "ZWNobyAiaGVsbG8gd29ybGQi",
                 "preserve_ephemeral": False
             }
         }
@@ -446,6 +447,7 @@ class TestServer(base.TestCase):
                                        min_microversion='2.1',
                                        max_microversion='2.56')
         self.sess.get_endpoint_data.return_value = self.endpoint_data
+        self.sess.default_microversion = None
 
         image_id = sot.create_image(self.sess, name, metadata)
 
@@ -473,6 +475,7 @@ class TestServer(base.TestCase):
                                        min_microversion='2.1',
                                        max_microversion='2.56')
         self.sess.get_endpoint_data.return_value = self.endpoint_data
+        self.sess.default_microversion = None
 
         self.assertIsNone(self.resp.body, sot.create_image(self.sess, name))
 
@@ -646,6 +649,18 @@ class TestServer(base.TestCase):
         self.sess.post.assert_called_with(
             url, json=body, headers=headers, microversion=None)
 
+    def test_lock_with_options(self):
+        sot = server.Server(**EXAMPLE)
+
+        res = sot.lock(self.sess, locked_reason='Because why not')
+
+        self.assertIsNone(res)
+        url = 'servers/IDENTIFIER/action'
+        body = {'lock': {'locked_reason': 'Because why not'}}
+        headers = {'Accept': ''}
+        self.sess.post.assert_called_with(
+            url, json=body, headers=headers, microversion=None)
+
     def test_unlock(self):
         sot = server.Server(**EXAMPLE)
 
@@ -769,6 +784,20 @@ class TestServer(base.TestCase):
         self.sess.post.assert_called_with(
             url, json=body, headers=headers, microversion=None)
 
+    def test_unshelve_availability_zone(self):
+        sot = server.Server(**EXAMPLE)
+
+        res = sot.unshelve(self.sess, sot.availability_zone)
+
+        self.assertIsNone(res)
+        url = 'servers/IDENTIFIER/action'
+        body = {"unshelve": {
+            "availability_zone": sot.availability_zone
+        }}
+        headers = {'Accept': ''}
+        self.sess.post.assert_called_with(
+            url, json=body, headers=headers, microversion=None)
+
     def test_migrate(self):
         sot = server.Server(**EXAMPLE)
 
@@ -804,10 +833,56 @@ class TestServer(base.TestCase):
         self.sess.post.assert_called_with(
             url, json=body, headers=headers, microversion=None)
 
+    def test_get_console_url(self):
+        sot = server.Server(**EXAMPLE)
+
+        resp = mock.Mock()
+        resp.body = {'console': {'a': 'b'}}
+        resp.json = mock.Mock(return_value=resp.body)
+        resp.status_code = 200
+        self.sess.post.return_value = resp
+
+        res = sot.get_console_url(self.sess, 'novnc')
+        self.sess.post.assert_called_with(
+            'servers/IDENTIFIER/action',
+            json={'os-getVNCConsole': {'type': 'novnc'}},
+            headers={'Accept': ''}, microversion=None)
+        self.assertDictEqual(resp.body['console'], res)
+
+        sot.get_console_url(self.sess, 'xvpvnc')
+        self.sess.post.assert_called_with(
+            'servers/IDENTIFIER/action',
+            json={'os-getVNCConsole': {'type': 'xvpvnc'}},
+            headers={'Accept': ''}, microversion=None)
+
+        sot.get_console_url(self.sess, 'spice-html5')
+        self.sess.post.assert_called_with(
+            'servers/IDENTIFIER/action',
+            json={'os-getSPICEConsole': {'type': 'spice-html5'}},
+            headers={'Accept': ''}, microversion=None)
+
+        sot.get_console_url(self.sess, 'rdp-html5')
+        self.sess.post.assert_called_with(
+            'servers/IDENTIFIER/action',
+            json={'os-getRDPConsole': {'type': 'rdp-html5'}},
+            headers={'Accept': ''}, microversion=None)
+
+        sot.get_console_url(self.sess, 'serial')
+        self.sess.post.assert_called_with(
+            'servers/IDENTIFIER/action',
+            json={'os-getSerialConsole': {'type': 'serial'}},
+            headers={'Accept': ''}, microversion=None)
+
+        self.assertRaises(ValueError,
+                          sot.get_console_url,
+                          self.sess,
+                          'fake_type'
+                          )
+
     def test_live_migrate_no_force(self):
         sot = server.Server(**EXAMPLE)
 
-        class FakeEndpointData(object):
+        class FakeEndpointData:
             min_microversion = None
             max_microversion = None
         self.sess.get_endpoint_data.return_value = FakeEndpointData()
@@ -818,12 +893,12 @@ class TestServer(base.TestCase):
             self.sess, host='HOST2', force=False, block_migration=False)
         self.assertIn(
             "Live migration on this cloud implies 'force'",
-            six.text_type(ex))
+            str(ex))
 
     def test_live_migrate_no_microversion_force_true(self):
         sot = server.Server(**EXAMPLE)
 
-        class FakeEndpointData(object):
+        class FakeEndpointData:
             min_microversion = None
             max_microversion = None
         self.sess.get_endpoint_data.return_value = FakeEndpointData()
@@ -849,10 +924,11 @@ class TestServer(base.TestCase):
     def test_live_migrate_25(self):
         sot = server.Server(**EXAMPLE)
 
-        class FakeEndpointData(object):
+        class FakeEndpointData:
             min_microversion = '2.1'
             max_microversion = '2.25'
         self.sess.get_endpoint_data.return_value = FakeEndpointData()
+        self.sess.default_microversion = None
 
         res = sot.live_migrate(
             self.sess, host='HOST2', force=True, block_migration=False)
@@ -873,10 +949,11 @@ class TestServer(base.TestCase):
     def test_live_migrate_25_default_block(self):
         sot = server.Server(**EXAMPLE)
 
-        class FakeEndpointData(object):
+        class FakeEndpointData:
             min_microversion = '2.1'
             max_microversion = '2.25'
         self.sess.get_endpoint_data.return_value = FakeEndpointData()
+        self.sess.default_microversion = None
 
         res = sot.live_migrate(
             self.sess, host='HOST2', force=True, block_migration=None)
@@ -897,10 +974,11 @@ class TestServer(base.TestCase):
     def test_live_migrate_30(self):
         sot = server.Server(**EXAMPLE)
 
-        class FakeEndpointData(object):
+        class FakeEndpointData:
             min_microversion = '2.1'
             max_microversion = '2.30'
         self.sess.get_endpoint_data.return_value = FakeEndpointData()
+        self.sess.default_microversion = None
 
         res = sot.live_migrate(
             self.sess, host='HOST2', force=False, block_migration=False)
@@ -921,10 +999,11 @@ class TestServer(base.TestCase):
     def test_live_migrate_30_force(self):
         sot = server.Server(**EXAMPLE)
 
-        class FakeEndpointData(object):
+        class FakeEndpointData:
             min_microversion = '2.1'
             max_microversion = '2.30'
         self.sess.get_endpoint_data.return_value = FakeEndpointData()
+        self.sess.default_microversion = None
 
         res = sot.live_migrate(
             self.sess, host='HOST2', force=True, block_migration=None)
@@ -943,6 +1022,73 @@ class TestServer(base.TestCase):
         self.sess.post.assert_called_with(
             url, json=body, headers=headers, microversion='2.30')
 
+    def test_get_topology(self):
+        sot = server.Server(**EXAMPLE)
+
+        class FakeEndpointData:
+            min_microversion = '2.1'
+            max_microversion = '2.78'
+        self.sess.get_endpoint_data.return_value = FakeEndpointData()
+        self.sess.default_microversion = None
+
+        response = mock.Mock()
+
+        topology = {
+            "nodes": [
+                {
+                    "cpu_pinning": {
+                        "0": 0,
+                        "1": 5
+                    },
+                    "host_node": 0,
+                    "memory_mb": 1024,
+                    "siblings": [
+                        [
+                            0,
+                            1
+                        ]
+                    ],
+                    "vcpu_set": [
+                        0,
+                        1
+                    ]
+                },
+                {
+                    "cpu_pinning": {
+                        "2": 1,
+                        "3": 8
+                    },
+                    "host_node": 1,
+                    "memory_mb": 2048,
+                    "siblings": [
+                        [
+                            2,
+                            3
+                        ]
+                    ],
+                    "vcpu_set": [
+                        2,
+                        3
+                    ]
+                }
+            ],
+            "pagesize_kb": 4
+        }
+
+        response.status_code = 200
+        response.json.return_value = {
+            'topology': topology
+        }
+
+        self.sess.get.return_value = response
+
+        fetched_topology = sot.fetch_topology(self.sess)
+
+        url = 'servers/IDENTIFIER/topology'
+        self.sess.get.assert_called_with(url)
+
+        self.assertEqual(fetched_topology, topology)
+
     def test_get_security_groups(self):
         sot = server.Server(**EXAMPLE)
 
