@@ -9,15 +9,15 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-from testscenarios import load_tests_apply_scenarios as load_tests  # noqa
 
 from hashlib import sha1
-import mock
 import random
-import six
 import string
 import tempfile
 import time
+from unittest import mock
+
+from testscenarios import load_tests_apply_scenarios as load_tests  # noqa
 
 from openstack.object_store.v1 import account
 from openstack.object_store.v1 import container
@@ -39,11 +39,13 @@ class TestObjectStoreProxy(test_proxy_base.TestProxyBase):
             endpoint=self.endpoint, container=self.container)
 
     def test_account_metadata_get(self):
-        self.verify_head(self.proxy.get_account_metadata, account.Account)
+        self.verify_head(
+            self.proxy.get_account_metadata, account.Account,
+            method_args=[])
 
     def test_container_metadata_get(self):
         self.verify_head(self.proxy.get_container_metadata,
-                         container.Container, value="container")
+                         container.Container, method_args=["container"])
 
     def test_container_delete(self):
         self.verify_delete(self.proxy.delete_container,
@@ -58,15 +60,17 @@ class TestObjectStoreProxy(test_proxy_base.TestProxyBase):
             self.proxy.create_container,
             container.Container,
             method_args=['container_name'],
+            expected_args=[],
             expected_kwargs={'name': 'container_name', "x": 1, "y": 2, "z": 3})
 
     def test_object_metadata_get(self):
-        self._verify2("openstack.proxy.Proxy._head",
-                      self.proxy.get_object_metadata,
-                      method_args=['object'],
-                      method_kwargs={'container': 'container'},
-                      expected_args=[obj.Object, 'object'],
-                      expected_kwargs={'container': 'container'})
+        self._verify(
+            "openstack.proxy.Proxy._head",
+            self.proxy.get_object_metadata,
+            method_args=['object'],
+            method_kwargs={'container': 'container'},
+            expected_args=[obj.Object, 'object'],
+            expected_kwargs={'container': 'container'})
 
     def _test_object_delete(self, ignore):
         expected_kwargs = {
@@ -74,12 +78,13 @@ class TestObjectStoreProxy(test_proxy_base.TestProxyBase):
             "container": "name",
         }
 
-        self._verify2("openstack.proxy.Proxy._delete",
-                      self.proxy.delete_object,
-                      method_args=["resource"],
-                      method_kwargs=expected_kwargs,
-                      expected_args=[obj.Object, "resource"],
-                      expected_kwargs=expected_kwargs)
+        self._verify(
+            "openstack.proxy.Proxy._delete",
+            self.proxy.delete_object,
+            method_args=["resource"],
+            method_kwargs=expected_kwargs,
+            expected_args=[obj.Object, "resource"],
+            expected_kwargs=expected_kwargs)
 
     def test_object_delete(self):
         self._test_object_delete(False)
@@ -90,11 +95,12 @@ class TestObjectStoreProxy(test_proxy_base.TestProxyBase):
     def test_object_create_attrs(self):
         kwargs = {"name": "test", "data": "data", "container": "name"}
 
-        self._verify2("openstack.proxy.Proxy._create",
-                      self.proxy.upload_object,
-                      method_kwargs=kwargs,
-                      expected_args=[obj.Object],
-                      expected_kwargs=kwargs)
+        self._verify(
+            "openstack.proxy.Proxy._create",
+            self.proxy.upload_object,
+            method_kwargs=kwargs,
+            expected_args=[obj.Object],
+            expected_kwargs=kwargs)
 
     def test_object_create_no_container(self):
         self.assertRaises(TypeError, self.proxy.upload_object)
@@ -103,7 +109,7 @@ class TestObjectStoreProxy(test_proxy_base.TestProxyBase):
         kwargs = dict(container="container")
         self.verify_get(
             self.proxy.get_object, obj.Object,
-            value=["object"],
+            method_args=["object"],
             method_kwargs=kwargs,
             expected_kwargs=kwargs)
 
@@ -181,7 +187,7 @@ class TestObjectStoreProxy(test_proxy_base.TestProxyBase):
 
     def test_file_segment(self):
         file_size = 4200
-        content = ''.join(random.SystemRandom().choice(
+        content = ''.join(random.choice(
             string.ascii_uppercase + string.digits)
             for _ in range(file_size)).encode('latin-1')
         self.imagefile = tempfile.NamedTemporaryFile(delete=False)
@@ -247,12 +253,19 @@ class TestDownloadObject(base_test_object.BaseTestObject):
 class TestExtractName(TestObjectStoreProxy):
 
     scenarios = [
-        ('discovery', dict(url='/', parts=['account']))
+        ('discovery', dict(url='/', parts=['account'])),
+        ('endpoints', dict(url='/endpoints', parts=['endpoints'])),
+        ('container', dict(url='/AUTH_123/container_name',
+                           parts=['container'])),
+        ('object', dict(url='/container_name/object_name',
+                        parts=['object'])),
+        ('object_long', dict(url='/v1/AUTH_123/cnt/path/deep/object_name',
+                             parts=['object']))
     ]
 
     def test_extract_name(self):
 
-        results = self.proxy._extract_name(self.url)
+        results = self.proxy._extract_name(self.url, project_id='123')
         self.assertEqual(self.parts, results)
 
 
@@ -281,7 +294,7 @@ class TestTempURL(TestObjectStoreProxy):
         url = self.proxy.generate_temp_url(
             self.url, self.seconds, self.method, temp_url_key=self.key)
         key = self.key
-        if not isinstance(key, six.binary_type):
+        if not isinstance(key, bytes):
             key = key.encode('utf-8')
         self.assertEqual(url, self.expected_url)
         self.assertEqual(hmac_mock.mock_calls, [
@@ -309,10 +322,10 @@ class TestTempURL(TestObjectStoreProxy):
                 path, self.seconds, self.method,
                 temp_url_key=self.key, ip_range=ip_range)
             key = self.key
-            if not isinstance(key, six.binary_type):
+            if not isinstance(key, bytes):
                 key = key.encode('utf-8')
 
-            if isinstance(ip_range, six.binary_type):
+            if isinstance(ip_range, bytes):
                 ip_range_expected_url = (
                     expected_url + ip_range.decode('utf-8')
                 )
@@ -357,7 +370,7 @@ class TestTempURL(TestObjectStoreProxy):
         lt = time.localtime()
         expires = time.strftime(self.expires_iso8601_format[:-1], lt)
 
-        if not isinstance(self.expected_url, six.string_types):
+        if not isinstance(self.expected_url, str):
             expected_url = self.expected_url.replace(
                 b'1400003600', bytes(str(int(time.mktime(lt))),
                                      encoding='ascii'))
@@ -372,7 +385,7 @@ class TestTempURL(TestObjectStoreProxy):
         expires = time.strftime(self.short_expires_iso8601_format, lt)
         lt = time.strptime(expires, self.short_expires_iso8601_format)
 
-        if not isinstance(self.expected_url, six.string_types):
+        if not isinstance(self.expected_url, str):
             expected_url = self.expected_url.replace(
                 b'1400003600', bytes(str(int(time.mktime(lt))),
                                      encoding='ascii'))
@@ -393,12 +406,12 @@ class TestTempURL(TestObjectStoreProxy):
                                            temp_url_key=self.key,
                                            iso8601=True)
         key = self.key
-        if not isinstance(key, six.binary_type):
+        if not isinstance(key, bytes):
             key = key.encode('utf-8')
 
         expires = time.strftime(self.expires_iso8601_format,
                                 time.gmtime(1400003600))
-        if not isinstance(self.url, six.string_types):
+        if not isinstance(self.url, str):
             self.assertTrue(url.endswith(bytes(expires, 'utf-8')))
         else:
             self.assertTrue(url.endswith(expires))
@@ -429,7 +442,7 @@ class TestTempURL(TestObjectStoreProxy):
                 path, self.seconds, self.method, prefix=True,
                 temp_url_key=self.key)
             key = self.key
-            if not isinstance(key, six.binary_type):
+            if not isinstance(key, bytes):
                 key = key.encode('utf-8')
             self.assertEqual(url, expected_url)
             self.assertEqual(hmac_mock.mock_calls, [
@@ -448,7 +461,7 @@ class TestTempURL(TestObjectStoreProxy):
 
     @mock.patch('hmac.HMAC.hexdigest', return_value="temp_url_signature")
     def test_generate_absolute_expiry_temp_url(self, hmac_mock):
-        if isinstance(self.expected_url, six.binary_type):
+        if isinstance(self.expected_url, bytes):
             expected_url = self.expected_url.replace(
                 b'1400003600', b'2146636800')
         else:
